import numpy as np

from FourierFilter import Sinc_Filter

def shiftFourierDescriptor(descriptor, t):
	n = len(descriptor)
	if n == 0:
		return []
	
	nHalf = n / 2
	cSet = np.arange(-nHalf, -nHalf + n)
	sSet = descriptor * np.exp(1j * 2 * np.pi * (t / n) * cSet)

	return sSet

def inverseFourierDescriptor(descriptor, terms = -1, points = -1):
	m = points
	n = len(descriptor)

	if n == 0:
		return []

	if terms < 0:
		# Use all of the terms in the Fourier series.
		terms = n

	if m < 0:
		# Output the same number of points as there 
		# are coefficients.
		m = n

	# Make a copy, since we will be mutating the
	# array directly.
	descriptor = descriptor.copy()

	termsHalf = terms / 2
	nHalf = n / 2

	# Zero terms based on the desired number of terms.
	descriptor[: max(nHalf - termsHalf, 0)] = 0
	descriptor[min(nHalf + (terms - termsHalf), n) :] = 0

	if m > n:
		# Say that we have n = 4, with Fourier indices
		# -2 -1 0 1
		# Then we should add the elements in the brackets:
		# m = 5:      -2 -1 0 1 (2)   [1 left, 1 right]
		# m = 6: (-3) -2 -1 0 1 (2)   [1 left, 1 right]
		# m = 7: (-3) -2 -1 0 1 (2 3) [1 left, 2 right]
		# So the right side gets the left-over.

		# Say that we have n = 3, with Fourier indices
		# -1 0 1
		# Then we should add the elements in the brackets:
		# m = 4: (-2) -1 0 1     [1 left, 0 right]
		# m = 5: (-2) -1 0 1 (2) [1 left, 1 right]
		# m = 6: (-3, -2) -1 0 1 (2) [2 left, 1 right]
		# So the left side gets the left-over.

		zeros = m - n
		# Note: rounds down due to integer arithmetic.
		zerosLeft = zeros / 2
		# Right side gets the left-over.
		zerosRight = zeros - zerosLeft

		if (n & 1) == 1:
			# Left side gets the left-over.
			zerosLeft, zerosRight = zerosRight, zerosLeft

		# Oversample to m samples; add zero coefficients.
		descriptor = np.hstack(
			(np.zeros(zerosLeft),
			descriptor,
			np.zeros(zerosRight)))
	elif m < n:
		# Undersample to m samples; aliase coefficients.
		mHalf = m / 2
		mDescriptor = np.zeros(m, dtype=complex)
		for i in range(-nHalf, -nHalf + n):
			mDescriptor[(i + mHalf) % m] += descriptor[i + nHalf]
		descriptor = mDescriptor

	return np.fft.ifft(np.fft.ifftshift(descriptor)) * len(descriptor)

def fourierFilter(
	pointSet,
	filter = Sinc_Filter(),
	filterOrder = 2):

	n = len(pointSet)
	if n == 0:
		return []

	# Compute the Fourier series coefficients.
	fSet = np.fft.fft(pointSet)

	# Circularly shift the coefficients to
	# correspond to a linearly increasing phase
	# in the Fourier series.
	fSet = np.fft.fftshift(fSet)

	# Compute the corresponding Fourier-series indices.
	# n = 3:     -1, 0, 1
	# n = 4: -2, -1, 0, 1
	nHalf = n / 2
	cSet = np.arange(-nHalf, -nHalf + n)

	if filter and filterOrder > 0:
		# Filter out ringing artifacts and Gibbs phenomenon.
		fSet *= filter(cSet / float(nHalf))**filterOrder

	return np.fft.ifft(np.fft.ifftshift(fSet))

def fourierDescriptor(
	pointSet,
	scaling = True, 
	rotation = True,
	shift = True,
	translation = True):
	
	n = len(pointSet)
	if n == 0:
		return []

	# Compute the Fourier series coefficients.
	fSet = np.fft.fft(pointSet) / n

	# Circularly shift the coefficients to
	# correspond to a linearly increasing phase
	# in the Fourier series.
	fSet = np.fft.fftshift(fSet)

	# Compute the corresponding Fourier-series indices.
	# n = 3:     -1, 0, 1
	# n = 4: -2, -1, 0, 1
	nHalf = n / 2
	cSet = np.arange(-nHalf, -nHalf + n)

	if translation:
		# Make translation-invariant by
		# removing the constant component.
		# It is important to do this here; otherwise
		# the constant component can be selected as having
		# the largest absolute value.
		fSet[nHalf] = 0

	# Convert to polar form.
	if scaling:
		# Ignore the translation component when
		# considering the maximum absolute value.
		mean = fSet[nHalf]
		fSet[nHalf] = 0

		# Make scale-invariant by dividing by the
		# maximum absolute value.
		fSet /= np.absolute(fSet).max()

		# Revert the translation component.
		fSet[nHalf] = mean

	if n == 1:
		return fSet

	# First negative phase.
	beta = np.angle(fSet[nHalf - 1])

	# First positive phase.
	alpha = beta
	if n >= 3:
		alpha = np.angle(fSet[nHalf + 1])

	if rotation:
		# Make rotation-invariant by canceling the
		# average phase of the first components.
		angle = (alpha + beta) / 2
		fSet /= np.exp(1j * angle)

	if shift:
		# Make shift-invariant by canceling half
		# of the phase-difference between the first components.
		angle = (alpha - beta) / 2
		fSet /= np.exp(1j * cSet * angle)

	return fSet
