# Description: Fourier viewer
# Documentation: fractional_fourier.txt

import os
import sys
import numpy as np
np.seterr(divide='ignore', invalid='ignore')

from FourierDescriptor import *
from FourierFilter import *
from ArcLength import *

import FourierViewerUi

from math import *
from PyQt4.QtGui import *
from PyQt4.QtCore import *

class FourierModel(object):
	def __init__(self):
		self.frameRate = 50

		self.pointSet = []
		self.noiseSet = []
		self.noisySet = []
		self.arcSet = []
		self.fSet = []
		self.length = 1
		self.step = 0
		self.steps = self.frameRate;
		self.filter = lambda x : np.sinc(x)
		self.order = 3
		
		self.useArcLength = True
		self.arcLength = 1
		self.noise = 10
		self.terms = 16
		
		self.pathDetail = 10

	def recompute(self):
		self.noisySet = self.pointSet + np.array(self.noiseSet) * self.noise
		self.noisySet = self.noisySet.tolist()

		n = len(self.noisySet)

		if n > 0:
			self.arcSet = self.noisySet

			if self.filter and self.order > 0:
				self.arcSet = fourierFilter(
					self.arcSet,
					self.filter,
					self.order)

			if self.useArcLength:
				self.arcSet = parametrizeByArcLength(self.arcSet, step = self.arcLength)

			self.length = polygonLength(self.arcSet)
			self.fSet = fourierDescriptor(
				self.arcSet,
				rotation = False,
				scaling = False,
				shift = False,
				translation = False)
		else:
			self.fSet = []

class FourierViewer(FourierViewerUi.FourierViewerUi):
	def __init__(self):
		FourierViewerUi.FourierViewerUi.__init__(self)

		self.model = FourierModel()
		self.view.model = self.model

		nameSet = ['-', 'Lanczos', 'Sidi', 'Korobev', 'Kaiser']
		for name in nameSet:
			self.filterBox.addItem(name)

		self.filterBox.highlighted.connect(self.onSetFilter)
		self.filterBox.setCurrentIndex(nameSet.index('-'))

		self.orderBox.valueChanged.connect(self.onOrderChanged)
		self.shapeBox.valueChanged.connect(self.onShapeChanged)
		self.noiseBox.valueChanged.connect(self.onNoiseChanged)
		self.termsBox.valueChanged.connect(self.onTermsChanged)
		self.arcCheck.stateChanged.connect(self.onUseArcChanged)
		self.arcBox.valueChanged.connect(self.onArcChanged)

		self.onOrderChanged(self.orderBox.value())
		self.onShapeChanged(self.shapeBox.value())
		self.onNoiseChanged(self.noiseBox.value())
		self.onTermsChanged(self.termsBox.value())
		self.onUseArcChanged(self.arcCheck.checkState())
		self.onArcChanged(self.arcBox.value())

		self.timer = QTimer(self)
		self.timer.timeout.connect(self.animate)
		self.timer.start(1000.0 / self.model.frameRate)

		self.resize(800, 800)
		self.setFocusPolicy(Qt.StrongFocus)
		self.view.setFocus()

	def recompute(self):
		self.model.recompute()
		self.update()

	@pyqtSlot(int)
	def onOrderChanged(self, order):
		self.model.order = order
		self.recompute()

	@pyqtSlot(int)
	def onShapeChanged(self, shape):
		self.model.shape = shape
		self.onSetFilter(self.filterBox.currentIndex())

	@pyqtSlot(float)
	def onNoiseChanged(self, noise):
		self.model.noise = noise
		self.recompute()

	@pyqtSlot(float)
	def onTermsChanged(self, terms):
		self.model.terms = terms
		self.recompute()

	@pyqtSlot(float)
	def onArcChanged(self, arcLength):
		self.model.arcLength = arcLength
		self.recompute()

	@pyqtSlot(int)
	def onUseArcChanged(self, useArcLength):
		self.model.useArcLength = (useArcLength == Qt.Checked)
		self.recompute()

	@pyqtSlot(int)
	def onSetFilter(self, n):
		name = str(self.filterBox.itemText(n))

		filter = None

		if name == '-':
			filter = Cut_Filter()
			self.shapeBox.setEnabled(False)
			self.orderBox.setEnabled(False)
		elif name == 'Lanczos':
			filter = Sinc_Filter()
			self.shapeBox.setEnabled(False)
			self.orderBox.setEnabled(True)
		elif name == 'Sidi':
			filter = Sidi_Filter(self.model.shape)
			self.shapeBox.setEnabled(True)
			self.orderBox.setEnabled(True)
		elif name == 'Korobev':
			filter = Korobev_Filter(self.model.shape)
			self.shapeBox.setEnabled(True)
			self.orderBox.setEnabled(True)
		elif name == 'Kaiser':
			filter = Kaiser_Filter(self.model.shape)
			self.shapeBox.setEnabled(True)
			self.orderBox.setEnabled(True)

		self.model.filter = filter
		self.recompute()

	@pyqtSlot()
	def animate(self):
		self.model.step += 1
		self.update()

def main(argv):
	# Create application.
	app = QApplication(argv)

	# Create the window.
	fourierViewer = FourierViewer()
	fourierViewer.show()

	# Start the application.
	sys.exit(app.exec_())

if __name__ == '__main__':
	main(sys.argv)
